#include <cstdio>
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <string>
#include <cmath>
#include <vector>
#include <set>

using namespace std;

const int MaxN = 20200;
const int MaxM = 200200;

struct E
{
	int ind, u, v, c, d; // index, cvorovi, cena_materijal, putarina
	E() {}
	E(int _ind, int _u, int _v, int _c, int _d)
	{
		ind = _ind; u = _u; v = _v; c = _c; d = _d;
	}
};

int n, m;
E edges[MaxM];
long long d[MaxN];
vector<int> adj[MaxN];
vector<int> ADJ[MaxN];
int outInd[MaxN], solInd[MaxN];
bool outIndMark[MaxM];
int inDegree[MaxN];
bool mark[MaxN];

// Dijkstra strukture --------------------------------
struct cmp {
	bool operator () (const int& a, const int& b) {
		if (d[a] == d[b])
			return a < b;
		return (d[a] < d[b]);
	}
};

set<int, cmp> heap;

void Dijkstra(int start)
{
	for (int i = 1; i <= n; i++) d[i] = -1;
	d[start] = 0LL;
	heap.clear();
	heap.insert(start);

	while (!heap.empty())
	{
		int u = *heap.begin();
		heap.erase(heap.begin());
		for (int i = 0; i < (int)adj[u].size(); i++)
		{
			E edge = edges[ adj[u][i] ];
			int v = (edge.u == u ? edge.v : edge.u); 
			long long w = edge.d;
			if (d[v] == -1) 
			{
				d[v] = d[u] + w;
				heap.insert(v);
			}
			else
				if (d[u] + w < d[v])
				{
					heap.erase(v);
					d[v] = d[u] + w;
					heap.insert(v);
				}
		}
	}
}
// ---------------------------------------------------

// Provera resenja
void DFS(int u)
{
	for (int i = 0; i < (int)ADJ[u].size(); i++)
	{
		inDegree[ ADJ[u][i] ]--;
		if (inDegree[ ADJ[u][i] ] == 0) DFS(ADJ[u][i]);
	}
}

void DFS2(int u)
{
	mark[u] = true;
	for (int i = 0; i < (int)ADJ[u].size(); i++)
	{
		if (!mark[ ADJ[u][i] ]) DFS2(ADJ[u][i]);
	}
}

bool checkSolution(ifstream& outFile)
{
	// provera inputa
	if (outFile.eof()) 
		return false;
	int n1;
	outFile >> n1;
	if (n1 != n - 1)
		return false;

	for (int i = 1; i < n; i++)
	{
		if (outFile.eof())
			return false;
		outFile >> outInd[i];
	}

	// ogranicenja
	for (int i = 1; i < n; i++)
		if (outInd[i] < 1 || outInd[i] > m)
			return false;

	// ponavljanje
	for (int i = 1; i <= m; i++) outIndMark[i] = false;
	for (int i = 1; i < n; i++)
	{
		if (outIndMark[ outInd[i] ] == true) return false;
		outIndMark[ outInd[i] ] = true;
	}

	// optimalnost
	long long solVal = 0LL;
	long long outVal = 0LL;
	for (int i = 1; i < n; i++)
	{
		solVal = solVal + d[ edges[ solInd[i] ].u ] + d[ edges[ solInd[i] ].v ] + edges[ solInd[i] ].c;
		outVal = outVal + d[ edges[ outInd[i] ].u ] + d[ edges[ outInd[i] ].v ] + edges[ outInd[i] ].c;
	}
	if (solVal != outVal)
		return false;

	// redosled
	for (int i = 1; i < n; i++)
	{
		int u = edges[ outInd[i] ].u;
		int v = edges[ outInd[i] ].v;
		int w = edges[ outInd[i] ].d;
		if ((u != 1 && inDegree[u] == 0) || (v != 1 && inDegree[v] == 0))
			return false;
		if (d[u] + w == d[v])
		{
			inDegree[v]--;
			if (inDegree[v] == 0) DFS(v);
		}
		if (d[v] + w == d[u])
		{
			inDegree[u]--;
			if (inDegree[u] == 0) DFS(u);
		}
	}

	// da li je upste spannig tree
	for (int i = 1; i <= n; i++) mark[i] = false;
	for (int i = 1; i <= n; i++) ADJ[i].clear();
	for (int i = 1; i < n; i++)
	{
		int u = edges[ outInd[i] ].u;
		int v = edges[ outInd[i] ].v;
		ADJ[u].push_back(v);
		ADJ[v].push_back(u);
	}
	DFS2(1);
	for (int i = 1; i <= n; i++)
		if (!mark[i]) return false;

	return true;
}

int main(int argc, char *argv[])
{  

	FILE* inFile = fopen(argv[1], "r");
	fscanf(inFile, "%d%d", &n, &m);
	for (int i = 1; i <= m; i++)
	{
		fscanf(inFile, "%d%d%d%d", &edges[i].u, &edges[i].v, &edges[i].c, &edges[i].d);
		adj[ edges[i].u ].push_back(i);
		adj[ edges[i].v ].push_back(i);
	}
	fclose(inFile);
	
	Dijkstra(1);
	for (int i = 1; i <= n; i++) inDegree[i] = 0;
	for (int i = 1; i <= m; i++)
	{
		int u = edges[i].u, v = edges[i].v, w = edges[i].d;
		if (d[u] + w == d[v]) 
		{ 
			inDegree[v]++;
			ADJ[u].push_back(v);
		}
		if (d[v] + w == d[u]) 
		{
			inDegree[u]++;
			ADJ[v].push_back(u);
		}
	}

	FILE* solFile = fopen(argv[3], "r");
	int n1;
	fscanf(solFile, "%d", &n1);
	for (int i = 1; i <= n - 1; i++)
	{
		fscanf(solFile, "%d", &solInd[i]);
	}
	fclose(solFile);

	ifstream outFile(argv[2]);

	bool isOk = checkSolution(outFile);
  
	cout << (isOk?1:0);

	outFile.close();
	return 0;
}